@@ -4,54 +4,48 @@
 
 from __future__ import absolute_import, division, print_function
 
-import os
-import math
 import json
-import random
-import warnings
 import logging
+import math
+import os
+import random
+import statistics
+import warnings
 from collections import defaultdict
 from itertools import chain
-import statistics
 from multiprocessing import cpu_count
 
+import numpy as np
+from scipy.stats import mode, pearsonr
+from sklearn.metrics import (
+    confusion_matrix,
+    f1_score,
+    label_ranking_average_precision_score,
+    matthews_corrcoef,
+    mean_squared_error,
+)
+from tqdm.auto import tqdm, trange
+
+import pandas as pd
 import torch
 import torch.nn.functional as F
-import numpy as np
-import pandas as pd
-
-from scipy.stats import pearsonr, mode
-from sklearn.metrics import (
-    mean_squared_error,
-    matthews_corrcoef,
-    confusion_matrix,
-    label_ranking_average_precision_score,
-    f1_score,
-)
-from tensorboardX import SummaryWriter
-from tqdm.auto import trange, tqdm
-
-from torch.utils.data.distributed import DistributedSampler
-from torch.utils.data import DataLoader, RandomSampler, SequentialSampler, TensorDataset
-
-from transformers import AdamW, get_linear_schedule_with_warmup
-from transformers import (
-    WEIGHTS_NAME,
-    GPT2Tokenizer,
-    GPT2DoubleHeadsModel,
-    GPT2Config,
-    OpenAIGPTDoubleHeadsModel,
-    OpenAIGPTTokenizer,
-    OpenAIGPTConfig,
-)
-
-from simpletransformers.classification.classification_utils import (
-    InputExample,
-    convert_examples_to_features,
-)
-
+from simpletransformers.classification.classification_utils import InputExample, convert_examples_to_features
 from simpletransformers.config.global_args import global_args
 from simpletransformers.conv_ai.conv_ai_utils import get_dataset
+from tensorboardX import SummaryWriter
+from torch.utils.data import DataLoader, RandomSampler, SequentialSampler, TensorDataset
+from torch.utils.data.distributed import DistributedSampler
+from transformers import (
+    WEIGHTS_NAME,
+    AdamW,
+    GPT2Config,
+    GPT2DoubleHeadsModel,
+    GPT2Tokenizer,
+    OpenAIGPTConfig,
+    OpenAIGPTDoubleHeadsModel,
+    OpenAIGPTTokenizer,
+    get_linear_schedule_with_warmup,
+)
 
 try:
     import wandb
@@ -103,6 +97,7 @@ class ConvAIModel:
                 torch.cuda.manual_seed_all(args["manual_seed"])
 
         config_class, model_class, tokenizer_class = MODEL_CLASSES[model_type]
+        self.__dict__.update(kwargs)
 
         if use_cuda:
             if torch.cuda.is_available():
@@ -398,7 +393,10 @@ class ConvAIModel:
                             best_eval_metric = results[args["early_stopping_metric"]]
                             self._save_model(args["best_model_dir"], model=model, results=results)
                         if best_eval_metric and args["early_stopping_metric_minimize"]:
-                            if results[args["early_stopping_metric"]] - best_eval_metric < args["early_stopping_delta"]:
+                            if (
+                                results[args["early_stopping_metric"]] - best_eval_metric
+                                < args["early_stopping_delta"]
+                            ):
                                 best_eval_metric = results[args["early_stopping_metric"]]
                                 self._save_model(args["best_model_dir"], model=model, results=results)
                                 early_stopping_counter = 0
@@ -412,12 +410,17 @@ class ConvAIModel:
                                             logger.info(f" Early stopping patience: {args['early_stopping_patience']}")
                                     else:
                                         if verbose:
-                                            logger.info(f" Patience of {args['early_stopping_patience']} steps reached")
+                                            logger.info(
+                                                f" Patience of {args['early_stopping_patience']} steps reached"
+                                            )
                                             logger.info(" Training terminated.")
                                             train_iterator.close()
                                         return global_step, tr_loss / global_step
                         else:
-                            if results[args["early_stopping_metric"]] - best_eval_metric > args["early_stopping_delta"]:
+                            if (
+                                results[args["early_stopping_metric"]] - best_eval_metric
+                                > args["early_stopping_delta"]
+                            ):
                                 best_eval_metric = results[args["early_stopping_metric"]]
                                 self._save_model(args["best_model_dir"], model=model, results=results)
                                 early_stopping_counter = 0
@@ -431,7 +434,9 @@ class ConvAIModel:
                                             logger.info(f" Early stopping patience: {args['early_stopping_patience']}")
                                     else:
                                         if verbose:
-                                            logger.info(f" Patience of {args['early_stopping_patience']} steps reached")
+                                            logger.info(
+                                                f" Patience of {args['early_stopping_patience']} steps reached"
+                                            )
                                             logger.info(" Training terminated.")
                                             train_iterator.close()
                                         return global_step, tr_loss / global_step
@@ -447,7 +452,10 @@ class ConvAIModel:
 
             if args["evaluate_during_training"]:
                 results, _, _ = self.eval_model(
-                    eval_dataloader, verbose=verbose and args["evaluate_during_training_verbose"], silent=True, **kwargs
+                    eval_dataloader,
+                    verbose=verbose and args["evaluate_during_training_verbose"],
+                    silent=True,
+                    **kwargs,
                 )
 
                 self._save_model(output_dir_current, results=results)
@@ -591,6 +599,7 @@ class ConvAIModel:
             dataset_path,
             args["cache_dir"],
             process_count=process_count,
+            proxies=self.__dict__.get("proxies", None),
             evaluate=evaluate,
             no_cache=no_cache,
         )
@@ -689,7 +698,14 @@ class ConvAIModel:
         self._move_model_to_device()
 
         if not personality:
-            dataset = get_dataset(tokenizer, None, args["cache_dir"], process_count=process_count, interact=True)
+            dataset = get_dataset(
+                tokenizer,
+                None,
+                args["cache_dir"],
+                process_count=process_count,
+                proxies=self.__dict__.get("proxies", None),
+                interact=True,
+            )
             personalities = [dialog["personality"] for dataset in dataset.values() for dialog in dataset]
             personality = random.choice(personalities)
         else:
