@@ -1,57 +1,54 @@
 from __future__ import absolute_import, division, print_function
 
-import os
-import math
 import json
+import logging
+import math
+import os
 import random
 import warnings
-import logging
 from multiprocessing import cpu_count
 
-import torch
 import numpy as np
-import pandas as pd
-
 from scipy.stats import pearsonr
-from seqeval.metrics import precision_score, recall_score, f1_score, classification_report
-from tensorboardX import SummaryWriter
-from tqdm.auto import trange, tqdm
-
-from torch.nn import CrossEntropyLoss
-from torch.utils.data import DataLoader, RandomSampler, SequentialSampler, TensorDataset
-
-from transformers import AdamW, get_linear_schedule_with_warmup
-from transformers import (
-    WEIGHTS_NAME,
-    BertConfig,
-    BertForTokenClassification,
-    BertTokenizer,
-)
-from transformers import (
-    DistilBertConfig,
-    DistilBertForTokenClassification,
-    DistilBertTokenizer,
-)
-from transformers import RobertaConfig, RobertaForTokenClassification, RobertaTokenizer
-from transformers import (
-    XLMRobertaConfig,
-    XLMRobertaForTokenClassification,
-    XLMRobertaTokenizer,
-)
+from seqeval.metrics import classification_report, f1_score, precision_score, recall_score
+from tqdm.auto import tqdm, trange
 
+import pandas as pd
+import torch
+from simpletransformers.config.global_args import global_args
 from simpletransformers.ner.ner_utils import (
     InputExample,
     convert_examples_to_features,
+    get_examples_from_df,
     get_labels,
     read_examples_from_file,
-    get_examples_from_df,
 )
+from tensorboardX import SummaryWriter
+from torch.nn import CrossEntropyLoss
+from torch.utils.data import DataLoader, RandomSampler, SequentialSampler, TensorDataset
 from transformers import (
+    WEIGHTS_NAME,
+    AdamW,
+    BertConfig,
+    BertForTokenClassification,
+    BertTokenizer,
     CamembertConfig,
     CamembertForTokenClassification,
     CamembertTokenizer,
+    DistilBertConfig,
+    DistilBertForTokenClassification,
+    DistilBertTokenizer,
+    ElectraConfig,
+    ElectraForTokenClassification,
+    ElectraTokenizer,
+    RobertaConfig,
+    RobertaForTokenClassification,
+    RobertaTokenizer,
+    XLMRobertaConfig,
+    XLMRobertaForTokenClassification,
+    XLMRobertaTokenizer,
+    get_linear_schedule_with_warmup,
 )
-from simpletransformers.config.global_args import global_args
 
 try:
     import wandb
@@ -102,17 +99,34 @@ class NERModel:
             ]
         self.num_labels = len(self.labels)
 
+        self.args = {}
+        self.args = {"classification_report": False}
+        self.args.update(global_args)
+
+        if not use_cuda:
+            self.args["fp16"] = False
+
+        if args:
+            self.args.update(args)
+
         MODEL_CLASSES = {
             "bert": (BertConfig, BertForTokenClassification, BertTokenizer),
             "roberta": (RobertaConfig, RobertaForTokenClassification, RobertaTokenizer),
             "distilbert": (DistilBertConfig, DistilBertForTokenClassification, DistilBertTokenizer),
             "camembert": (CamembertConfig, CamembertForTokenClassification, CamembertTokenizer),
             "xlmroberta": (XLMRobertaConfig, XLMRobertaForTokenClassification, XLMRobertaTokenizer),
+            "electra": (ElectraConfig, ElectraForTokenClassification, ElectraTokenizer),
         }
 
         config_class, model_class, tokenizer_class = MODEL_CLASSES[model_type]
+        if self.num_labels:
+            self.config = config_class.from_pretrained(model_name, num_labels=self.num_labels, **self.args["config"])
+            self.num_labels = self.num_labels
+        else:
+            self.config = config_class.from_pretrained(model_name, **self.args["config"])
+            self.num_labels = self.config.num_labels
 
-        self.model = model_class.from_pretrained(model_name, num_labels=self.num_labels, **kwargs)
+        self.model = model_class.from_pretrained(model_name, config=self.config, **kwargs)
 
         if use_cuda:
             if torch.cuda.is_available():
@@ -130,17 +144,9 @@ class NERModel:
 
         self.results = {}
 
-        self.args = {}
-        self.args = {"classification_report": False}
-        self.args.update(global_args)
-
-        if not use_cuda:
-            self.args["fp16"] = False
-
-        if args:
-            self.args.update(args)
-
-        self.tokenizer = tokenizer_class.from_pretrained(model_name, do_lower_case=self.args["do_lower_case"], **kwargs)
+        self.tokenizer = tokenizer_class.from_pretrained(
+            model_name, do_lower_case=self.args["do_lower_case"], **kwargs
+        )
 
         self.args["model_name"] = model_name
         self.args["model_type"] = model_type
@@ -403,9 +409,14 @@ class NERModel:
 
                         if not best_eval_metric:
                             best_eval_metric = results[args["early_stopping_metric"]]
-                            self._save_model(args["best_model_dir"], optimizer, scheduler, model=model, results=results)
+                            self._save_model(
+                                args["best_model_dir"], optimizer, scheduler, model=model, results=results
+                            )
                         if best_eval_metric and args["early_stopping_metric_minimize"]:
-                            if results[args["early_stopping_metric"]] - best_eval_metric < args["early_stopping_delta"]:
+                            if (
+                                results[args["early_stopping_metric"]] - best_eval_metric
+                                < args["early_stopping_delta"]
+                            ):
                                 best_eval_metric = results[args["early_stopping_metric"]]
                                 self._save_model(
                                     args["best_model_dir"], optimizer, scheduler, model=model, results=results
@@ -421,12 +432,17 @@ class NERModel:
                                             logger.info(f" Early stopping patience: {args['early_stopping_patience']}")
                                     else:
                                         if verbose:
-                                            logger.info(f" Patience of {args['early_stopping_patience']} steps reached")
+                                            logger.info(
+                                                f" Patience of {args['early_stopping_patience']} steps reached"
+                                            )
                                             logger.info(" Training terminated.")
                                             train_iterator.close()
                                         return global_step, tr_loss / global_step
                         else:
-                            if results[args["early_stopping_metric"]] - best_eval_metric > args["early_stopping_delta"]:
+                            if (
+                                results[args["early_stopping_metric"]] - best_eval_metric
+                                > args["early_stopping_delta"]
+                            ):
                                 best_eval_metric = results[args["early_stopping_metric"]]
                                 self._save_model(
                                     args["best_model_dir"], optimizer, scheduler, model=model, results=results
@@ -442,7 +458,9 @@ class NERModel:
                                             logger.info(f" Early stopping patience: {args['early_stopping_patience']}")
                                     else:
                                         if verbose:
-                                            logger.info(f" Patience of {args['early_stopping_patience']} steps reached")
+                                            logger.info(
+                                                f" Patience of {args['early_stopping_patience']} steps reached"
+                                            )
                                             logger.info(" Training terminated.")
                                             train_iterator.close()
                                         return global_step, tr_loss / global_step
@@ -479,11 +497,40 @@ class NERModel:
                         best_eval_metric = results[args["early_stopping_metric"]]
                         self._save_model(args["best_model_dir"], optimizer, scheduler, model=model, results=results)
                         early_stopping_counter = 0
+                    else:
+                        if args["use_early_stopping"] and args["early_stopping_consider_epochs"]:
+                            if early_stopping_counter < args["early_stopping_patience"]:
+                                early_stopping_counter += 1
+                                if verbose:
+                                    logger.info(f" No improvement in {args['early_stopping_metric']}")
+                                    logger.info(f" Current step: {early_stopping_counter}")
+                                    logger.info(f" Early stopping patience: {args['early_stopping_patience']}")
+                            else:
+                                if verbose:
+                                    logger.info(f" Patience of {args['early_stopping_patience']} steps reached")
+                                    logger.info(" Training terminated.")
+                                    train_iterator.close()
+                                return global_step, tr_loss / global_step
                 else:
                     if results[args["early_stopping_metric"]] - best_eval_metric > args["early_stopping_delta"]:
                         best_eval_metric = results[args["early_stopping_metric"]]
                         self._save_model(args["best_model_dir"], optimizer, scheduler, model=model, results=results)
                         early_stopping_counter = 0
+                        early_stopping_counter = 0
+                    else:
+                        if args["use_early_stopping"] and args["early_stopping_consider_epochs"]:
+                            if early_stopping_counter < args["early_stopping_patience"]:
+                                early_stopping_counter += 1
+                                if verbose:
+                                    logger.info(f" No improvement in {args['early_stopping_metric']}")
+                                    logger.info(f" Current step: {early_stopping_counter}")
+                                    logger.info(f" Early stopping patience: {args['early_stopping_patience']}")
+                            else:
+                                if verbose:
+                                    logger.info(f" Patience of {args['early_stopping_patience']} steps reached")
+                                    logger.info(" Training terminated.")
+                                    train_iterator.close()
+                                return global_step, tr_loss / global_step
 
         return global_step, tr_loss / global_step
 
