@@ -1,65 +1,68 @@
 from __future__ import absolute_import, division, print_function
 
-import os
-import math
 import json
 import logging
+import math
+import os
 import random
 import warnings
-
 from multiprocessing import cpu_count
 
-import torch
 import numpy as np
-import pandas as pd
-
 from scipy.stats import pearsonr
 from sklearn.metrics import (
-    mean_squared_error,
-    matthews_corrcoef,
     confusion_matrix,
     label_ranking_average_precision_score,
+    matthews_corrcoef,
+    mean_squared_error,
+)
+from tqdm.auto import tqdm, trange
+
+import pandas as pd
+import torch
+from simpletransformers.config.global_args import global_args
+from simpletransformers.custom_models.models import ElectraForQuestionAnswering
+from simpletransformers.question_answering.question_answering_utils import (
+    RawResult,
+    RawResultExtended,
+    build_examples,
+    convert_examples_to_features,
+    get_best_predictions,
+    get_best_predictions_extended,
+    get_examples,
+    to_list,
+    write_predictions,
+    write_predictions_extended,
 )
 from tensorboardX import SummaryWriter
-from tqdm.auto import trange, tqdm
-
-from torch.utils.data.distributed import DistributedSampler
 from torch.utils.data import DataLoader, RandomSampler, SequentialSampler, TensorDataset
-
-from transformers import AdamW, get_linear_schedule_with_warmup
+from torch.utils.data.distributed import DistributedSampler
 from transformers import (
     WEIGHTS_NAME,
+    AdamW,
+    AlbertConfig,
+    AlbertForQuestionAnswering,
+    AlbertTokenizer,
     BertConfig,
     BertForQuestionAnswering,
     BertTokenizer,
+    DistilBertConfig,
+    DistilBertForQuestionAnswering,
+    DistilBertTokenizer,
+    ElectraConfig,
+    ElectraTokenizer,
+    RobertaConfig,
+    RobertaForQuestionAnswering,
+    RobertaTokenizer,
     XLMConfig,
     XLMForQuestionAnswering,
     XLMTokenizer,
     XLNetConfig,
     XLNetForQuestionAnswering,
     XLNetTokenizer,
-    DistilBertConfig,
-    DistilBertForQuestionAnswering,
-    DistilBertTokenizer,
-    AlbertConfig,
-    AlbertForQuestionAnswering,
-    AlbertTokenizer,
+    get_linear_schedule_with_warmup,
 )
 
-from simpletransformers.question_answering.question_answering_utils import (
-    get_examples,
-    convert_examples_to_features,
-    RawResult,
-    write_predictions,
-    RawResultExtended,
-    write_predictions_extended,
-    to_list,
-    build_examples,
-    get_best_predictions,
-    get_best_predictions_extended,
-)
-from simpletransformers.config.global_args import global_args
-
 try:
     import wandb
 
@@ -80,7 +83,8 @@ class QuestionAnsweringModel:
         Args:
             model_type: The type of model (bert, xlnet, xlm, distilbert)
             model_name: Default Transformer model name or path to a directory containing Transformer model file (pytorch_nodel.bin).
-            args (optional): Default args will be used if this parameter is not provided. If provided, it should be a dict containing the args that should be changed in the default args['
+            args (optional): Default args will be used if this parameter is not provided. If provided,
+                it should be a dict containing the args that should be changed in the default args'
             use_cuda (optional): Use GPU if available. Setting to False will force model to use CPU only.
             cuda_device (optional): Specific GPU that should be used. Will use the first available GPU by default.
         """  # noqa: ignore flake8"
@@ -91,6 +95,8 @@ class QuestionAnsweringModel:
             "xlm": (XLMConfig, XLMForQuestionAnswering, XLMTokenizer),
             "distilbert": (DistilBertConfig, DistilBertForQuestionAnswering, DistilBertTokenizer,),
             "albert": (AlbertConfig, AlbertForQuestionAnswering, AlbertTokenizer),
+            "roberta": (RobertaConfig, RobertaForQuestionAnswering, RobertaTokenizer),
+            "electra": (ElectraConfig, ElectraForQuestionAnswering, ElectraTokenizer),
         }
 
         if args and "manual_seed" in args:
@@ -100,25 +106,6 @@ class QuestionAnsweringModel:
             if "n_gpu" in args and args["n_gpu"] > 0:
                 torch.cuda.manual_seed_all(args["manual_seed"])
 
-        config_class, model_class, tokenizer_class = MODEL_CLASSES[model_type]
-        self.model = model_class.from_pretrained(model_name, **kwargs)
-
-        if use_cuda:
-            if torch.cuda.is_available():
-                if cuda_device == -1:
-                    self.device = torch.device("cuda")
-                else:
-                    self.device = torch.device(f"cuda:{cuda_device}")
-            else:
-                raise ValueError(
-                    "'use_cuda' set to True when cuda is unavailable."
-                    " Make sure CUDA is available or set use_cuda=False."
-                )
-        else:
-            self.device = "cpu"
-
-        self.results = {}
-
         self.args = {
             "doc_stride": 384,
             "max_query_length": 64,
@@ -138,7 +125,29 @@ class QuestionAnsweringModel:
         if args:
             self.args.update(args)
 
-        self.tokenizer = tokenizer_class.from_pretrained(model_name, do_lower_case=self.args["do_lower_case"], **kwargs)
+        config_class, model_class, tokenizer_class = MODEL_CLASSES[model_type]
+        self.config = config_class.from_pretrained(model_name, **self.args["config"])
+        self.model = model_class.from_pretrained(model_name, config=self.config, **kwargs)
+
+        if use_cuda:
+            if torch.cuda.is_available():
+                if cuda_device == -1:
+                    self.device = torch.device("cuda")
+                else:
+                    self.device = torch.device(f"cuda:{cuda_device}")
+            else:
+                raise ValueError(
+                    "'use_cuda' set to True when cuda is unavailable."
+                    " Make sure CUDA is available or set use_cuda=False."
+                )
+        else:
+            self.device = "cpu"
+
+        self.results = {}
+
+        self.tokenizer = tokenizer_class.from_pretrained(
+            model_name, do_lower_case=self.args["do_lower_case"], **kwargs
+        )
 
         self.args["model_name"] = model_name
         self.args["model_type"] = model_type
@@ -222,7 +231,7 @@ class QuestionAnsweringModel:
         return dataset
 
     def train_model(
-        self, train_data, output_dir=False, show_running_loss=True, args=None, eval_data=None, verbose=True
+        self, train_data, output_dir=False, show_running_loss=True, args=None, eval_data=None, verbose=True, **kwargs
     ):
         """
         Trains the model using 'train_data'
@@ -232,7 +241,10 @@ class QuestionAnsweringModel:
             output_dir: The directory where model files will be saved. If not given, self.args['output_dir'] will be used.
             show_running_loss (optional): Set to False to prevent running loss from being printed to console. Defaults to True.
             args (optional): Optional changes to the args dict of the model. Any changes made will persist for the model.
-            eval_data (optional): Path to JSON file containing evaluation data against which evaluation will be performed when evaluate_during_training is enabled. Is required if evaluate_during_training is enabled.
+            eval_data (optional): Path to JSON file containing evaluation data against which evaluation will be performed when evaluate_during_training is enabled.
+                Is required if evaluate_during_training is enabled.
+            **kwargs: Additional metrics that should be used. Pass in the metrics as keyword arguments (name of metric: function to use).
+                A metric function should take in two parameters. The first parameter will be the true labels, and the second parameter will be the predictions.
         Returns:
             None
         """  # noqa: ignore flake8"
@@ -271,7 +283,7 @@ class QuestionAnsweringModel:
         os.makedirs(output_dir, exist_ok=True)
 
         global_step, tr_loss = self.train(
-            train_dataset, output_dir, show_running_loss=show_running_loss, eval_data=eval_data,
+            train_dataset, output_dir, show_running_loss=show_running_loss, eval_data=eval_data, **kwargs
         )
 
         model_to_save = self.model.module if hasattr(self.model, "module") else self.model
@@ -281,7 +293,7 @@ class QuestionAnsweringModel:
 
         logger.info(" Training of {} model complete. Saved to {}.".format(self.args["model_type"], output_dir))
 
-    def train(self, train_dataset, output_dir, show_running_loss=True, eval_data=None, verbose=True):
+    def train(self, train_dataset, output_dir, show_running_loss=True, eval_data=None, verbose=True, **kwargs):
         """
         Trains the model on train_dataset.
 
@@ -361,7 +373,7 @@ class QuestionAnsweringModel:
                 logger.info("   Starting fine-tuning.")
 
         if args["evaluate_during_training"]:
-            training_progress_scores = self._create_training_progress_scores()
+            training_progress_scores = self._create_training_progress_scores(**kwargs)
 
         if args["wandb_project"]:
             wandb.init(project=args["wandb_project"], config={**args}, **args["wandb_kwargs"])
@@ -447,7 +459,7 @@ class QuestionAnsweringModel:
                         and global_step % args["evaluate_during_training_steps"] == 0
                     ):
                         # Only evaluate when single GPU otherwise metrics may not average well
-                        results, _ = self.eval_model(eval_data, verbose=True)
+                        results, _ = self.eval_model(eval_data, verbose=False, **kwargs)
                         for key, value in results.items():
                             tb_writer.add_scalar("eval_{}".format(key), value, global_step)
 
@@ -470,9 +482,14 @@ class QuestionAnsweringModel:
 
                         if not best_eval_metric:
                             best_eval_metric = results[args["early_stopping_metric"]]
-                            self._save_model(args["best_model_dir"], optimizer, scheduler, model=model, results=results)
+                            self._save_model(
+                                args["best_model_dir"], optimizer, scheduler, model=model, results=results
+                            )
                         if best_eval_metric and args["early_stopping_metric_minimize"]:
-                            if results[args["early_stopping_metric"]] - best_eval_metric < args["early_stopping_delta"]:
+                            if (
+                                results[args["early_stopping_metric"]] - best_eval_metric
+                                < args["early_stopping_delta"]
+                            ):
                                 best_eval_metric = results[args["early_stopping_metric"]]
                                 self._save_model(
                                     args["best_model_dir"], optimizer, scheduler, model=model, results=results
@@ -488,12 +505,17 @@ class QuestionAnsweringModel:
                                             logger.info(f" Early stopping patience: {args['early_stopping_patience']}")
                                     else:
                                         if verbose:
-                                            logger.info(f" Patience of {args['early_stopping_patience']} steps reached")
+                                            logger.info(
+                                                f" Patience of {args['early_stopping_patience']} steps reached"
+                                            )
                                             logger.info(" Training terminated.")
                                             train_iterator.close()
                                         return global_step, tr_loss / global_step
                         else:
-                            if results[args["early_stopping_metric"]] - best_eval_metric > args["early_stopping_delta"]:
+                            if (
+                                results[args["early_stopping_metric"]] - best_eval_metric
+                                > args["early_stopping_delta"]
+                            ):
                                 best_eval_metric = results[args["early_stopping_metric"]]
                                 self._save_model(
                                     args["best_model_dir"], optimizer, scheduler, model=model, results=results
@@ -509,7 +531,9 @@ class QuestionAnsweringModel:
                                             logger.info(f" Early stopping patience: {args['early_stopping_patience']}")
                                     else:
                                         if verbose:
-                                            logger.info(f" Patience of {args['early_stopping_patience']} steps reached")
+                                            logger.info(
+                                                f" Patience of {args['early_stopping_patience']} steps reached"
+                                            )
                                             logger.info(" Training terminated.")
                                             train_iterator.close()
                                         return global_step, tr_loss / global_step
@@ -524,7 +548,7 @@ class QuestionAnsweringModel:
                 self._save_model(output_dir_current, optimizer, scheduler, model=model)
 
             if args["evaluate_during_training"]:
-                results, _ = self.eval_model(eval_data, verbose=True)
+                results, _ = self.eval_model(eval_data, verbose=False, **kwargs)
 
                 self._save_model(output_dir_current, optimizer, scheduler, results=results)
 
@@ -546,15 +570,43 @@ class QuestionAnsweringModel:
                         best_eval_metric = results[args["early_stopping_metric"]]
                         self._save_model(args["best_model_dir"], optimizer, scheduler, model=model, results=results)
                         early_stopping_counter = 0
+                    else:
+                        if args["use_early_stopping"] and args["early_stopping_consider_epochs"]:
+                            if early_stopping_counter < args["early_stopping_patience"]:
+                                early_stopping_counter += 1
+                                if verbose:
+                                    logger.info(f" No improvement in {args['early_stopping_metric']}")
+                                    logger.info(f" Current step: {early_stopping_counter}")
+                                    logger.info(f" Early stopping patience: {args['early_stopping_patience']}")
+                            else:
+                                if verbose:
+                                    logger.info(f" Patience of {args['early_stopping_patience']} steps reached")
+                                    logger.info(" Training terminated.")
+                                    train_iterator.close()
+                                return global_step, tr_loss / global_step
                 else:
                     if results[args["early_stopping_metric"]] - best_eval_metric > args["early_stopping_delta"]:
                         best_eval_metric = results[args["early_stopping_metric"]]
                         self._save_model(args["best_model_dir"], optimizer, scheduler, model=model, results=results)
                         early_stopping_counter = 0
+                    else:
+                        if args["use_early_stopping"] and args["early_stopping_consider_epochs"]:
+                            if early_stopping_counter < args["early_stopping_patience"]:
+                                early_stopping_counter += 1
+                                if verbose:
+                                    logger.info(f" No improvement in {args['early_stopping_metric']}")
+                                    logger.info(f" Current step: {early_stopping_counter}")
+                                    logger.info(f" Early stopping patience: {args['early_stopping_patience']}")
+                            else:
+                                if verbose:
+                                    logger.info(f" Patience of {args['early_stopping_patience']} steps reached")
+                                    logger.info(" Training terminated.")
+                                    train_iterator.close()
+                                return global_step, tr_loss / global_step
 
         return global_step, tr_loss / global_step
 
-    def eval_model(self, eval_data, output_dir=None, verbose=False):
+    def eval_model(self, eval_data, output_dir=None, verbose=False, verbose_logging=False, **kwargs):
         """
         Evaluates the model on eval_data. Saves results to output_dir.
 
@@ -562,6 +614,9 @@ class QuestionAnsweringModel:
             eval_data: Path to JSON file containing evaluation data OR list of Python dicts in the correct format. The model will be evaluated on this data.
             output_dir: The directory where model files will be saved. If not given, self.args['output_dir'] will be used.
             verbose: If verbose, results will be printed to the console on completion of evaluation.
+            verbose_logging: Log info related to feature conversion and writing predictions.
+            **kwargs: Additional metrics that should be used. Pass in the metrics as keyword arguments (name of metric: function to use).
+                A metric function should take in two parameters. The first parameter will be the true labels, and the second parameter will be the predictions.
 
         Returns:
             result: Dictionary containing evaluation results. (correct, similar, incorrect)
@@ -573,7 +628,9 @@ class QuestionAnsweringModel:
 
         self._move_model_to_device()
 
-        all_predictions, all_nbest_json, scores_diff_json = self.evaluate(eval_data, output_dir)
+        all_predictions, all_nbest_json, scores_diff_json, eval_loss = self.evaluate(
+            eval_data, output_dir, verbose_logging=verbose
+        )
 
         if isinstance(eval_data, str):
             with open(eval_data, "r", encoding=self.args["encoding"]) as f:
@@ -581,7 +638,8 @@ class QuestionAnsweringModel:
         else:
             truth = eval_data
 
-        result, texts = self.calculate_results(truth, all_predictions)
+        result, texts = self.calculate_results(truth, all_predictions, **kwargs)
+        result["eval_loss"] = eval_loss
 
         self.results.update(result)
 
@@ -590,7 +648,7 @@ class QuestionAnsweringModel:
 
         return result, texts
 
-    def evaluate(self, eval_data, output_dir):
+    def evaluate(self, eval_data, output_dir, verbose_logging=False):
         """
         Evaluates the model on eval_data.
 
@@ -614,6 +672,8 @@ class QuestionAnsweringModel:
         eval_sampler = SequentialSampler(eval_dataset)
         eval_dataloader = DataLoader(eval_dataset, sampler=eval_sampler, batch_size=args["eval_batch_size"])
 
+        eval_loss = 0.0
+        nb_eval_steps = 0
         model.eval()
 
         all_results = []
@@ -624,10 +684,11 @@ class QuestionAnsweringModel:
                 inputs = {
                     "input_ids": batch[0],
                     "attention_mask": batch[1],
+                    "token_type_ids": batch[2],
                 }
 
-                if args["model_type"] != "distilbert":
-                    inputs["token_type_ids"] = None if args["model_type"] == "xlm" else batch[2]
+                if self.args["model_type"] in ["xlm", "roberta", "distilbert", "camembert", "electra"]:
+                    del inputs["token_type_ids"]
 
                 example_indices = batch[3]
 
@@ -635,6 +696,7 @@ class QuestionAnsweringModel:
                     inputs.update({"cls_index": batch[4], "p_mask": batch[5]})
 
                 outputs = model(**inputs)
+                eval_loss += outputs[0].mean().item()
 
                 for i, example_index in enumerate(example_indices):
                     eval_feature = features[example_index.item()]
@@ -651,10 +713,16 @@ class QuestionAnsweringModel:
                         )
                     else:
                         result = RawResult(
-                            unique_id=unique_id, start_logits=to_list(outputs[0][i]), end_logits=to_list(outputs[1][i]),
+                            unique_id=unique_id,
+                            start_logits=to_list(outputs[0][i]),
+                            end_logits=to_list(outputs[1][i]),
                         )
                     all_results.append(result)
 
+            nb_eval_steps += 1
+
+        eval_loss = eval_loss / nb_eval_steps
+
         prefix = "test"
         os.makedirs(output_dir, exist_ok=True)
 
@@ -678,7 +746,7 @@ class QuestionAnsweringModel:
                 model.config.end_n_top,
                 True,
                 tokenizer,
-                not args["silent"],
+                verbose_logging,
             )
         else:
             all_predictions, all_nbest_json, scores_diff_json = write_predictions(
@@ -691,12 +759,12 @@ class QuestionAnsweringModel:
                 output_prediction_file,
                 output_nbest_file,
                 output_null_log_odds_file,
-                not args["silent"],
+                verbose_logging,
                 True,
                 args["null_score_diff_threshold"],
             )
 
-        return all_predictions, all_nbest_json, scores_diff_json
+        return all_predictions, all_nbest_json, scores_diff_json, eval_loss
 
     def predict(self, to_predict, n_best_size=None):
         """
@@ -746,10 +814,11 @@ class QuestionAnsweringModel:
                 inputs = {
                     "input_ids": batch[0],
                     "attention_mask": batch[1],
+                    "token_type_ids": batch[2],
                 }
 
-                if args["model_type"] != "distilbert":
-                    inputs["token_type_ids"] = None if args["model_type"] == "xlm" else batch[2]
+                if self.args["model_type"] in ["xlm", "roberta", "distilbert", "camembert", "electra"]:
+                    del inputs["token_type_ids"]
 
                 example_indices = batch[3]
 
@@ -773,7 +842,9 @@ class QuestionAnsweringModel:
                         )
                     else:
                         result = RawResult(
-                            unique_id=unique_id, start_logits=to_list(outputs[0][i]), end_logits=to_list(outputs[1][i]),
+                            unique_id=unique_id,
+                            start_logits=to_list(outputs[0][i]),
+                            end_logits=to_list(outputs[1][i]),
                         )
                     all_results.append(result)
 
@@ -797,10 +868,9 @@ class QuestionAnsweringModel:
 
         return answers
 
-    def calculate_results(self, truth, predictions):
+    def calculate_results(self, truth, predictions, **kwargs):
         truth_dict = {}
         questions_dict = {}
-        logger.info(truth)
         for item in truth:
             for answer in item["qas"]:
                 if answer["answers"]:
@@ -815,8 +885,12 @@ class QuestionAnsweringModel:
         correct_text = {}
         incorrect_text = {}
         similar_text = {}
+        predicted_answers = []
+        true_answers = []
 
         for q_id, answer in truth_dict.items():
+            predicted_answers.append(predictions[q_id])
+            true_answers.append(answer)
             if predictions[q_id].strip() == answer.strip():
                 correct += 1
                 correct_text[q_id] = answer
@@ -835,11 +909,11 @@ class QuestionAnsweringModel:
                     "question": questions_dict[q_id],
                 }
 
-        result = {
-            "correct": correct,
-            "similar": similar,
-            "incorrect": incorrect,
-        }
+        extra_metrics = {}
+        for metric, func in kwargs.items():
+            extra_metrics[metric] = func(true_answers, predicted_answers)
+
+        result = {"correct": correct, "similar": similar, "incorrect": incorrect, **extra_metrics}
 
         texts = {
             "correct_text": correct_text,
@@ -859,24 +933,29 @@ class QuestionAnsweringModel:
         inputs = {
             "input_ids": batch[0],
             "attention_mask": batch[1],
+            "token_type_ids": batch[2],
             "start_positions": batch[3],
             "end_positions": batch[4],
         }
 
-        if self.args["model_type"] != "distilbert":
-            inputs["token_type_ids"] = None if self.args["model_type"] == "xlm" else batch[2]
+        if self.args["model_type"] in ["xlm", "roberta", "distilbert", "camembert", "electra"]:
+            del inputs["token_type_ids"]
+
         if self.args["model_type"] in ["xlnet", "xlm"]:
             inputs.update({"cls_index": batch[5], "p_mask": batch[6]})
 
         return inputs
 
-    def _create_training_progress_scores(self):
+    def _create_training_progress_scores(self, **kwargs):
+        extra_metrics = {key: [] for key in kwargs}
         training_progress_scores = {
             "global_step": [],
             "correct": [],
             "similar": [],
             "incorrect": [],
             "train_loss": [],
+            "eval_loss": [],
+            **extra_metrics,
         }
 
         return training_progress_scores
